@@ -2,6 +2,7 @@
 SILE = {}
 
 SILE.version = require("core.version")
+SILE.features = require("core.features")
 
 -- Initialize Lua environment and global utilities
 SILE.lua_version = _VERSION:sub(-3)
@@ -74,7 +75,8 @@ SILE.shapers = core_loader("shapers")
 SILE.outputters = core_loader("outputters")
 SILE.classes = core_loader("classes")
 SILE.packages = core_loader("packages")
--- SILE.typesetters = core_loader("typesetters")
+SILE.typesetters = core_loader("typesetters")
+SILE.pagebuilders = core_loader("pagebuilders")
 
 -- Internal libraries that don't make assumptions on load, only use
 SILE.traceStack = require("core.tracestack")()
@@ -109,13 +111,8 @@ local function runEvals (evals, arg)
 end
 
 SILE.init = function ()
-  -- Set by def
   if not SILE.backend then
-    if pcall(require, "justenoughharfbuzz") then
-      SILE.backend = "libtexpdf"
-    else
-      SU.error("Default backend libtexpdf not available!")
-    end
+    SILE.backend = "libtexpdf"
   end
   if SILE.backend == "libtexpdf" then
     SILE.shaper = SILE.shapers.harfbuzz()
@@ -133,6 +130,7 @@ SILE.init = function ()
     SILE.shaper = SILE.shapers.harfbuzz()
     SILE.outputter = SILE.outputters.dummy()
   end
+  SILE.pagebuilder = SILE.pagebuilders.base()
   runEvals(SILE.input.evaluates, "evaluate")
 end
 
@@ -165,6 +163,9 @@ SILE.use = function (module, options)
   elseif pack.type == "typesetter" then
     SILE.typesetters[name] = pack
     SILE.typesetter = pack(options)
+  elseif pack.type == "pagebuilder" then
+    SILE.pagebuilders[name] = pack
+    SILE.pagebuilder = pack(options)
   elseif pack.type == "package" then
     SILE.packages[name] = pack
     if class then
@@ -190,7 +191,14 @@ SILE.require = function (dependency, pathprefix, deprecation_ack)
   if pathprefix then
     status, lib = pcall(require, pl.path.join(pathprefix, dependency))
   end
-  if not status then lib = require(dependency) end
+  if not status then
+    local prefixederror = lib
+    status, lib = pcall(require, dependency)
+    if not status then
+      SU.error(("Unable to find module '%s'%s")
+        :format(dependency, SILE.traceback and ((pathprefix and "\n  " .. prefixederror or "") .. "\n  " .. lib) or ""))
+    end
+  end
   local class = SILE.documentState.documentClass
   if not class and not deprecation_ack then
     SU.warn(string.format([[
@@ -252,7 +260,7 @@ local function detectFormat (doc, filename)
   local initialround = filename and 1 or 2
   for round = initialround, 3 do
     for _, inputter in ipairs(contentDetectionOrder) do
-      SU.debug("inputter", ("Running content type detection round %s with %s"):format(round, inputter._name))
+      SU.debug("inputter", "Running content type detection round", round, "with", inputter._name)
       if inputter.appropriate(round, filename, doc) then
         return inputter._name
       end
@@ -276,7 +284,9 @@ function SILE.processString (doc, format, filename, options)
     inputter = SILE.inputter
   else
     format = format or detectFormat(doc, filename)
-    io.stderr:write(("<%s> as %s\n"):format(SILE.currentlyProcessingFile, format))
+    if not SILE.quiet then
+      io.stderr:write(("<%s> as %s\n"):format(SILE.currentlyProcessingFile, format))
+    end
     inputter = SILE.inputters[format](options)
     -- If we did content detection *and* this is the master file, save the
     -- inputter for posterity and postambles
@@ -314,13 +324,29 @@ function SILE.processFile (filename, format, options)
     end
     doc = file:read("*a")
   end
+  local cpf = SILE.currentlyProcessingFile
   SILE.currentlyProcessingFile = filename
   local pId = SILE.traceStack:pushDocument(filename, doc)
   local ret = SILE.processString(doc, format, filename, options)
   SILE.traceStack:pop(pId)
+  SILE.currentlyProcessingFile = cpf
   return ret
 end
 
+-- TODO: this probably needs deprecating, moved here just to get out of the way so
+-- typesetters classing works as expected
+SILE.typesetNaturally = function (frame, func)
+  local saveTypesetter = SILE.typesetter
+  if SILE.typesetter.frame then SILE.typesetter.frame:leave(SILE.typesetter) end
+  SILE.typesetter = SILE.typesetters.base(frame)
+  SILE.settings:temporarily(func)
+  SILE.typesetter:leaveHmode()
+  SILE.typesetter:chuck()
+  SILE.typesetter.frame:leave(SILE.typesetter)
+  SILE.typesetter = saveTypesetter
+  if SILE.typesetter.frame then SILE.typesetter.frame:enter(SILE.typesetter) end
+end
+
 -- Sort through possible places files could be
 function SILE.resolveFile (filename, pathprefix)
   local candidates = {}
@@ -406,14 +432,15 @@ function SILE.finish ()
     SILE.makeDeps:write()
   end
   SILE.documentState.documentClass:finish()
+  SILE.font.finish()
   runEvals(SILE.input.evaluateAfters, "evaluate-after")
-  io.stderr:write("\n")
+  if not SILE.quiet then
+    io.stderr:write("\n")
+  end
 end
 
 -- Internal libraries that run core SILE functions on load
 SILE.settings = require("core.settings")()
-SILE.pagebuilder = require("core.pagebuilder")()
-require("typesetters.base")
 require("core.hyphenator-liang")
 require("core.languages")
 require("core.packagemanager")
